-module(futam).
-compile(export_all).
-author('hanak@iit.bme.hu, modified by: kapolnai@iit.bme.hu').
-vsn('$LastChangedDate: 2020-10-05 10:02:09 +0200 (h, 05 okt 2020) $$').

%%% Első változat: futam és maradék előállítása két függvénnyel

-type elem() :: any().
-type pred() :: fun((elem(), elem()) -> boolean()).

-spec elso_futam(P::pred(), Ls::[elem()]) -> Fs::[elem()].
%% Fs az Ls P-t kielégítő első, tovább nem bővíthető futama (prefixuma).
elso_futam(_P, [X]) ->
    [X];
elso_futam(P, [X|[Y|_]=Ys]) ->
    case P(X, Y) of
        true  -> [X|elso_futam(P, Ys)];
        false -> [X]
    end.

-spec maradek(P::pred(), Ls::[elem()]) -> Ms::[elem()].
%% Ms az Ls P-t kielégítő első futam utáni maradéka.
maradek(_P, [_X]) ->
    [];
maradek(P, [X|[Y|_]=Ys]) ->
    case P(X, Y) of
        true  -> maradek(P, Ys);
        false -> Ys
    end.

-spec naiv_futamok(Pred::pred(), Ls::[elem()]) -> Lss::list([elem()]).
%% Lss az Ls szomszédos, Pred-et kielégítő elemeiből álló, tovább
%% nem bővíthető, diszjunkt futamok listája.
naiv_futamok(_P, []) ->
    [];
naiv_futamok(P, Ls) ->
    Fs = elso_futam(P, Ls),
    Ms = maradek(P, Ls),
    [ Fs | naiv_futamok(P, Ms) ].

%%% Második változat: futam és maradék előállítása egy függvénnyel

%% elso_futam(P, L) az L P-t kielégítő első (prefix) futama.
%% maradek(P, L) az L P-t kielégítő első futam utáni maradéka.
-spec futam_maradek(P::pred(), L::[elem()]) -> {Fs::[elem()],Ms::[elem()]}.
%% Fs =:= elso_futam(P, L), Ms =:= maradek(P, L).
futam_maradek(_P, [X]) ->
    {[X], []};
futam_maradek(P, [X|Ys=[Y|_]]) ->
    case P(X, Y) of
        true ->
            {Fs, Ms} = futam_maradek(P, Ys),
            {[X|Fs], Ms};
        false ->
            {[X], Ys}
    end.

%% futamok(P, L) =:= naiv_futamok(P, L).
futamok(_P, []) ->
    [];
futamok(P, Ls) ->
    {Fs, Ms} = futam_maradek(P, Ls),
    [Fs|futamok(P, Ms)].

%%% Példa
test() ->
    L = [1,3,9,5,7,2,5,9,1,6,0,0,3,5,6,2],
    L2 = [1,9,19,3,4,24,34,4,11,45,66,13,45,66,99],
    [
     naiv_futamok(fun erlang:'<'/2, L),
     futam_maradek(fun erlang:'<'/2, L),
     futamok(fun erlang:'<'/2, L),
     futamok(fun erlang:'>='/2, L),
     futamok(fun(A,B) -> 2*A>B end, L),
     elso_futam(fun erlang:'=<'/2, L2), %%% [1,9,19]
     maradek(fun erlang:'=<'/2, L2), %%% [3,4,24,34,4,11,45,66,13,45,66,99].
     naiv_futamok(fun erlang:'=<'/2, L2), %%% [[1,9,19], [3,4,24,34], [4,11,45,66], [13,45,66,99]].
     naiv_futamok(fun(A,B) -> A=<B end, [299,1]), %%% [[299], [1]].
     naiv_futamok(fun(A,B) -> A=<B end, [299]), %%% [[299]].
     naiv_futamok(fun(A,B) -> A=<B end, []), %%% [].
     difek([1,3,5,7,7,5,3,1,1,1,1,2])
    ].


-spec difek(Xs::[number()]) -> Dss::[[number()]].
%% A Dss lista az Xs lista számtani sorozatot alkotó részlistáinak listája.
difek([X1,X2|_]=Xs) ->
    {Fs,Ms} = futam_maradek(fun(A, B) -> B-A =:= X2-X1 end, Xs),
    [Fs|difek(Ms)];
difek([_]=Xs) ->
    [Xs];
difek([]) ->
    [].
