-module(khf).
%-compile(export_all).
-export([elofordul0/2,
         elofordul1/2,
         elofordul1b/2,
         elofordul1c/2,
         elofordul1d/2,
         elofordul1e/2,
         filter/2,
         kiadott_kishazik/1,
         kiadott_kishazik2/1,
         megfelelt/2,
         bizt_kiadott_kishazik/1,
         bizt_teljesitmeny/2,
         teszt/0]).
-author('kapolnai@iit.bme.hu, hanak@emt.bme.hu').
-vsn('$LastChangedDate: 2019-10-29 21:47:55 +0100 (k, 29 okt 2019) $$').

-spec kiadott_kishazik(Ny::atom()) -> Db::integer().
% Az Ny nyelven Db darab kisházit adtak ki.
kiadott_kishazik(cekla)  -> 1;
kiadott_kishazik(prolog) -> 3;
kiadott_kishazik(erlang) -> 3.

-spec elofordul0(E::term(), Ls::[term()]) -> N::integer().
% E elem az Ls listában N-szer fordul elő.
% Fordítási figyelmeztetést (warning) kapunk!
elofordul0(Elem, [])           -> 0;
elofordul0(Elem, [Elem|Farok]) -> 1 + elofordul0(Elem, Farok);
elofordul0(Elem, [Fej|Farok])  -> elofordul0(Elem, Farok).

-spec elofordul1(E::term(), Ls::[term()]) -> N::integer().
% E elem az Ls listában N-szer fordul elő.
% Itt már nincs figyelmeztetés.
elofordul1(_Elem, [])          -> 0;
elofordul1(Elem, [Elem|Farok]) -> 1 + elofordul1(Elem, Farok);
elofordul1(Elem, [_Fej|Farok]) -> elofordul1(Elem, Farok).

-type hallgato() :: {nev(), [ {kovetelmenytipus(), munka()} ] }.
-type nev() :: atom().
-type kovetelmenytipus() :: atom().
-type munka() :: [atom()] | integer().
-spec megfelelt(K::kovetelmenytipus(), H::hallgato()) -> B::boolean().
%% B igaz, ha H hallgató megfelelt a K követelménynek.
megfelelt(khf, {_Nev, [{khf, L}|_]}) ->
    C = elofordul1(cekla, L),
    P = elofordul1(prolog, L),
    E = elofordul1(erlang, L),
    (P >= 1) and (E >= 1) and (C + P + E >= 3);
megfelelt(zh, {_Nev, [{zh, Pont}|_]}) ->
    Pont >= 24; 
megfelelt(K, {Nev, [_|F]}) ->
    megfelelt(K, {Nev, F});
megfelelt(_, {_, []}) ->
    false.

-spec elofordul1b(E::term(), Ls::[term()]) -> N::integer().
% E elem az Ls listában N-szer fordul elő.
% Itt sincs figyelmeztetés.
elofordul1b(_, [])              -> 0;
elofordul1b(Elem, [Elem|Farok]) -> 1 + elofordul1b(Elem, Farok);
elofordul1b(Elem, [_|Farok])    -> elofordul1b(Elem, Farok).

-spec elofordul1c(E::term(), Ls::[term()]) -> N::integer().
% E elem az Ls listában N-szer fordul elő.
% Itt sincs figyelmeztetés.
elofordul1c(_, []) ->
    0;
elofordul1c(Elem, [Fej|Farok]) ->
    case Fej of
        Elem -> 1;
        _    -> 0
    end
	+ elofordul1c(Elem, Farok).

-spec elofordul1d(E::term(), Ls::[term()]) -> N::integer().
% E elem az Ls listában N-szer fordul elő.
elofordul1d(Elem, L) ->
    length(filter(fun(X) -> X=:=Elem end, L)).

-spec elofordul1e(E::term(), Ls::[term()]) -> N::integer().
% E elem az Ls listában N-szer fordul elő.
elofordul1e(_, []) ->
    0;
elofordul1e(Elem, [Fej|Farok]) ->
    if 
        Fej =:= Elem -> 1;
        true         -> 0
    end
        + elofordul1e(Elem, Farok).

-spec bizt_kiadott_kishazik(Ny::atom()) -> {ok, Db::integer()} | error.
%% Az Ny nyelven Db darab kisházit adtak ki.
bizt_kiadott_kishazik(cekla)  -> {ok, 1};
bizt_kiadott_kishazik(prolog) -> {ok, 3};
bizt_kiadott_kishazik(erlang) -> {ok, 3};
bizt_kiadott_kishazik(_Ny)    -> error.

% bizt_kiadott_kishazik/1 felhasználása, pl. teljesítés %-ban:
-spec bizt_teljesitmeny(Nyelv::atom(), Beadott_db::integer()) -> {ok, Teljesitmeny::float()} | error.
bizt_teljesitmeny(Nyelv, Beadott_db) ->
    case bizt_kiadott_kishazik(Nyelv) of
        {ok, Kiadott_db} -> {ok, Beadott_db / Kiadott_db};
        error            -> error
    end.
    
% kiadott_kishazik/1 case-zel:
-spec kiadott_kishazik2(Ny::atom()) -> Db::integer().
% Az Ny nyelven Db darab kisházit adtak ki.
kiadott_kishazik2(Ny) ->
    case Ny of
        cekla  -> 1;
        prolog -> 3;
        erlang -> 3
    end.

-spec filter(P::fun((term()) -> boolean()), Ls::[term()]) -> F::[term()].
%% Az Ls lista P-t kielégítő elemeinek listája F.
filter(_, []) ->
    [];
filter(P, [Fej|Farok]) ->
    case P(Fej) of
        true  -> [Fej|filter(P,Farok)];
        false -> filter(P,Farok)
    end.

teszt() ->
    Hallgato1 = {'Diak Detti',
                  [{khf, [cekla,prolog,erlang,prolog]},
                   {zh, 59}]},
    Hallgato2 = {'Lusta Ludvig', []},
    [{N,K,megfelelt(K,{N,A})} || K <- [khf,zh], {N,A} <- [Hallgato1, Hallgato2]].
